#! /usr/local/bin/perl
#  cat-pie.pl - Graphs a pie chart of categories
#
#  Modified by Arlindo L. Oliveira (aml@inesc.pt)
#
#  Copyright (C) 1994 - 1999  Curtis L. Olson  - curt@me.umn.edu
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# cat-pie.pl.in,v 0.11 2001/07/13 04:05:59 briank Exp

# specify the installed location of the necessary pieces.
my($cbbpath) = "/usr/local/lib/cbb-0.9";
@INC = ("$cbbpath/perl", @INC);
package CBB;

use strict;    # don't take no guff

my($tmp, $temp);
my($key, $date, $check, $desc, $debit, $credit, $cat, $com, $cleared, $total);
my($credit_total, $debit_total, $amt, $lkey, $lcat, $subtotal);
my($tcom, $tamt, $tcat);
my(@keys, %ALLTRANS, @splits);
my($graphpath, $account, $name, $result);

$graphpath = "$cbbpath/graphs";

require "common.pl";
require "reports.pl";
require "engine.pl";
require "memorized.pl";

if ($#ARGV < 0) {
    die "Usage: report [ -from yyyymmdd ] [ -to yyyymmdd ] path accounts\n";
}

# process arguments
my($fromdate, $todate, @account_list) = &process_rep_args();

foreach $account ( @account_list ) {
    if ( ! -r $account) {
	die "Cannot read $account\n";
    }
}
if ( ! -x "$graphpath/graphbal") {
    die "Cannot launch $graphpath/graphbal\n";
}

%ALLTRANS = ();

# load all matching transactions from all specified accounts (ignoring
# those that are outside the specified date range)

my(%tmp_cat) = ();

foreach $account ( @account_list ) {
    $name = &file_basename($account);

    # open the account
    (&load_trans($account) eq "ok") || die "Cannot open account:  $account";

    $result = &first_trans();
    while ( $result ne "none" ) {
        ($key, $date, $check, $desc, $debit, $credit, $cat, $com, $cleared,
         $total) = split(/\t/, $result);

        $amt = $credit - $debit;

        if ( (($fromdate == 0) || ($fromdate <= $date)) && 
            (($todate == 0) || ($todate >= $date)) ) {

            $ALLTRANS{"$key$name"} = $result;

            if ( substr($cat, 0, 1) ne "|" ) {
                $tmp_cat{$cat} .= "$key$name" . "~" . $amt . "~";
            } else {
                # process split

                @splits = split(/\|/, $cat);
                shift(@splits);

                $tmp = 0;
                while ( $#splits >= 0 ) {
                    $tcat = shift(@splits);
                    $tcom = shift(@splits);
                    $tamt = shift(@splits);

                    $tmp += $tamt;

                    # print "processing $tcat $tamt\n";
                    $tmp_cat{$tcat} .= "$key$name" . "~" . $tamt . "~";
                }
                if ( sprintf("%.2f", $tmp) ne sprintf("%.2f", $amt) ) {
                    printf("WARNING:  Incorrect splits in $date: $desc\n");
                    printf("    %.2f != %.2f\n\n", $tmp, $amt);
                }
            }
        }

        $result = &next_trans();
    }
}


$credit_total = 0.00;
$debit_total = 0.00;

if ( ! -x "$graphpath/graphpie") {
    die "Cannot launch $graphpath/graphpie\n";
}

open(DATA,"| $graphpath/graphpie") || die "Cannot launch graph\n";

foreach $lcat (sort keys(%tmp_cat)) {
    
    chop($tmp_cat{$lcat});  # Delete final separator

    @keys = split(/~/, $tmp_cat{$lcat});

    $subtotal = 0.00;

    while ( $#keys >= 0 ) {
        $lkey = shift(@keys);
        $amt  = shift(@keys);

        $result = $ALLTRANS{$lkey};

        ($key, $date, $check, $desc, $debit, $credit, $cat, $com, $cleared,
         $total) = split(/\t/, $result);

        $subtotal = $subtotal + $amt;
        if ( $amt > 0 ) {
            $credit_total = $credit_total + $amt;
        } else {
            $debit_total = $debit_total + $amt;
        }
    }

    $lcat =~ s/ /-/g;
    $lcat eq "" and $lcat="<empty>";
    print DATA "$lcat $subtotal\n" if $subtotal;
    # printf(" = %9.2f\n", $subtotal);
}


# printf("                                      ---------\n" );
# printf("                      Total Credits = %9.2f\n\n", $credit_total);

# printf("                                         ---------\n" );
# printf("                       Total Debits = %9.2f\n\n", $debit_total);

# printf("                                         ---------\n" );
# printf("                            Balance = %9.2f\n\n", 
#       $credit_total + $debit_total);

close(DATA);
