#! /usr/local/bin/perl
#  txn-list.pl - Graphs running balance.
#
#  Modified by Arlindo L. Oliveira (aml@inesc.pt)
#
#  Copyright (C) 1994 - 1999  Curtis L. Olson  - curt@me.umn.edu
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# txn-list.pl.in,v 0.11 2001/07/13 04:05:59 briank Exp

# specify the installed location of the necessary pieces.
my($cbbpath) = "/usr/local/lib/cbb-0.9";
@INC = ("$cbbpath/perl", @INC);
package CBB;

use strict;    # don't take no guff

my($account, @account_list, $name, $key, $trans, %ALLTRANS);
my($gtotal, $junk, $result);
my($date, $date_fmt);
my($key, $date, $check, $desc, $debit, $credit, $cat, $com, $cleared, $total);
my($credit_total, $debit_total, $amt, $lkey, $lcat, $subtotal, $priorbalance);
my($trans, $remaining, $gtotal);
my(@keys, %ALLTRANS, @splits);
my($graphpath);
my($delta, $ldate);

$graphpath = "$cbbpath/graphs";

require "common.pl";
require "reports.pl";
require "engine.pl";
require "memorized.pl";

if ($#ARGV < 0) {
    die "Usage: report [ -from yyyymmdd ] [ -to yyyymmdd ] path accounts\n";
}

# process arguments
my($fromdate, $todate, @account_list) = &process_rep_args();

foreach $account ( @account_list ) {
    if ( ! -r $account) {
	die "Cannot read $account\n";
    }
}
if ( ! -x "$graphpath/graphbal") {
    die "Cannot launch $graphpath/graphbal\n";
}

%ALLTRANS = ();

# load all matching transactions from all specified accounts (ignoring
# those that are outside the specified date range)

$priorbalance = 0;
foreach $account ( @account_list ) {
    $name = &file_basename($account);

    # open the account
    (&load_trans($account) eq "ok") || die "Cannot open account:  $account";

    $result = &first_trans();
    while ( $result ne "none" ) {
        ($key, $trans) = split(/\t/, $result, 2);
	($date, $remaining) = split(/\t/, $trans, 2);

        if (($fromdate <= $date) && ($todate >= $date)) {
            $ALLTRANS{"$key$name"} = $trans;
        }
        if ($fromdate && ($fromdate > $date)) {
	    ($date, $check, $desc, $debit, $credit, $cat, $com, $cleared, $total) = split(/\t/, $trans);
	    $priorbalance += $credit - $debit;
        }
        $result = &next_trans();
    }
}

# sort and print

open(DATA, "| $graphpath/graphbal") || die "Cannot launch graph\n";

# print "prior balance = $priorbalance\n";
$gtotal = $priorbalance;

$ldate = 0;
$delta = 0;
foreach $key (sort (keys %ALLTRANS) ) {
#    print $ALLTRANS{$key} ;
    ($date, $check, $desc, $debit, $credit, $cat, $com, $cleared, $total) =
	split(/\t/, $ALLTRANS{$key});

    if ($ldate == $date) {
	$delta += $credit - $debit;
	next;
    }
    
    $ldate = $date;
    $gtotal += $credit - $debit + $delta;
    $delta = 0;
#    print " $gtotal \n";

    print DATA "$date $gtotal\n";
#    print "$date $gtotal\n";
}

close(DATA);

# print "Done with Graph\n";
