#!/usr/bin/wish -f
#  'CBB' -- Check Book Balancer
#
#   init.tcl -- Initialization routines.
#
#  Written by Curtis Olson.  Started December 7, 1996.
#
#  Copyright (C) 1994 - 1999  Curtis L. Olson  - curt@me.umn.edu
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# init.tcl,v 0.11 2001/08/12 20:47:36 briank Exp


# ----------------------------------------------------------------------------
# Draw splash screen on startup
# ----------------------------------------------------------------------------

proc cbb_splash {} {                                              
    global lib_path

    toplevel .x -bd 3 -relief raised
    wm withdraw .x
    set sw [winfo screenwidth .]
    set sh [winfo screenheight .]
    image create photo "splash" -file "$lib_path/images/splash-32.gif"
    wm overrideredirect .x 1
    label .x.l -image splash -bd 1 -relief sunken
    pack .x.l -side top -expand 1 -fill both
    set x [expr ($sw - 200)/2]
    set y [expr ($sh - 250)/2]
    wm geometry .x +$x+$y
    wm deiconify .x
    update idletasks
    after 500 {destroy .x}
}


# ----------------------------------------------------------------------------
# Initialize CBB
# ----------------------------------------------------------------------------

proc cbb_setup {} {
    global tcl_precision cbb eng lib_path

    # lets get this out of the way right away ... increases the precision.
    # but perhaps too much?  this is 1 millipenny on 1 trillion dollars 
    set tcl_precision 17

    # lets start with a "clean" slate
    acctSetClean
    reset_desc_memory
    reset_split_memory

    # open a two way pipe to the perl engine.
    set eng [open |$lib_path/cbbdb r+]

    set cbb(date_now) [clock format [clock seconds] -format "%4Y%02m%02d"]
    set_date_format $cbb(date_fmt)

    # Set debugging flag on/off in perl engine
    puts $eng "debug $cbb(debug)"; flush $eng
    gets $eng result
    if { $cbb(debug) } { puts $result }

    # Set cache flag on/off in perl engine
    puts $eng "cache $cbb(cache)"; flush $eng
    gets $eng result
    if { $cbb(debug) } { puts $result }

    # If Crypt is activated, ask for password
    if { $cbb(use_crypt) } { useCrypt }

    # Set duplicate entries flag in perl engine
    puts $eng "duplicate $cbb(duplicate)"; flush $eng
    gets $eng result
    if { $cbb(debug) } { puts $result }

    # Set duplicate compare fields flag in perl engine
    puts $eng "compare $cbb(cmpfields)"; flush $eng
    gets $eng result
    if { $cbb(debug) } { puts $result }

    # parse the command line
    parse_command_line

    # setup the main window
    setup_main

    # load account if ~/.cbbcur.tcl was parsed during cbb init routine
    if {  [info exists cbb(new_group)] && $cbb(new_group) != "" } {
      load_account "$cbb(new_group)" "$cbb(cur_path)/$cbb(new_group)/$cbb(cur_file)"
    }

    # make sure we have a backup file being saved ... automatically
    setup_auto_save

    # initialize main window bindings
    setup_default_binding

    # initialize undo
    undoInit

    # Remind user to remove ~/.cbbrc if it still exists.
    if { [file exists "~/.cbbrc"] } {
	cbbWindow.ok "You can remove your ``.cbbrc'' file because it is \
		obsolete.  It has been replaced with a ``.cbbrc.tcl'' file."
	tkwait window .ok
    }

    # If the user doesn't have a ~/.cbbrc.tcl file, create one for him.
    if { [file exists $cbb(conf_file)] } {
	# it exists, don't bother right now
    } else {
	cbbrcSave
    }

    # This should remain the last thing in the setup procedure.  It leaves 
    # a welcome message at the bottom of the window.

    setMainInfo
    .status.line configure -text "Welcome to the Check Book Balancer."

    update
}


# ----------------------------------------------------------------------------
# Initialize CBB Variables
# ----------------------------------------------------------------------------

proc cbb_init_language { filename } {
    global words lib_path

    source $lib_path/languages/${filename}.tcl
}


# ----------------------------------------------------------------------------
# Initialize CBB Variables
# ----------------------------------------------------------------------------

proc cbb_init_vars {} {
    global cbb env

    # Set global variables
    set cbb(author_xbm) "author.xbm"
    set cbb(conf_file) "$env(HOME)/.cbbrc.tcl"
    set cbb(save_cur_file) "$env(HOME)/.cbbcur.tcl"

    # we are going to store this in the internal format - EAM
    set cbb(cur_date) ""
    set cbb(cur_name) ""
    set cbb(cur_desc) ""
    set cbb(cur_file) ""
    # No harm done if the path doesn't already exist...
    file mkdir "$env(HOME)/.cbb"
    set cbb(cur_path) "$env(HOME)/.cbb"
    set cbb(cur_group) ""
    set cbb(cur_cats) ""
    set cbb(def_cat) "default.cat"
    set cbb(devel) 0
    set cbb(help_text) "cbb-man.txt"
    set cbb(icon_xbm) "cbb.xbm"
    set cbb(index1) 0
    set cbb(index2) 0
    set cbb(next_chk) 0

    set cbb(cat_entry) 0
    set cbb(cat_match) ""
    set cbb(cat_partial) ""
    set cbb(cat_found) 0

    set cbb(safe_mode) 0
    set cbb(selected) 0
    set cbb(state_start) 0.00
    set cbb(state_end) 0.00
    set cbb(version) "Version 0.9.0"

    # Preferences changed
    set cbb(pref_chg) 0

    # internal Startdate
    set cbb(int_sdate) "0"

    # code to use for cryptography
    set cbb(crypt_code) ""

    # The following defaults are over ridden by $cbb(CONF_FILE)

    set cbb(auto_hilite) 1
    set cbb(button_font) "-Adobe-Helvetica-Bold-R-Normal-*-*-120-*-*-*-*-*-*"
    set cbb(debug) 0
    set cbb(default_font) "-Adobe-Helvetica-Medium-R-Normal-*-*-120-*-*-*-*-*-*"
    set cbb(dialog_font) "-Adobe-Helvetica-Bold-R-Normal-*-*-120-*-*-*-*-*-*"
    set cbb(fixed_font) "8x13"
    set cbb(fixed_header_font) "8x13"
    set cbb(head_color) #00008f
    set cbb(hilite_color) #8080b0
    set cbb(list_line1_color) ""
    set cbb(list_line2_color) grey78
    set cbb(trans_text_color) black
    set cbb(trans_neg_color) red
    set cbb(future_color) grey30
    set cbb(future_neg_color) orange
    set cbb(list_height) 28
    set cbb(list_width) 72
    set cbb(acctlist_height) 5
    set cbb(max_splits) 16
    set cbb(menu_font) "-Adobe-Helvetica-Medium-R-Normal-*-*-120-*-*-*-*-*-*"
    set cbb(msg_text_font) "-Adobe-Helvetica-Bold-R-Normal-*-*-120-*-*-*-*-*-*"
    # 1000 milleseconds = 1 second (save_interval is specified in milleseconds)
    set cbb(save_interval) 180000
    set cbb(auto_save) 0
    set cbb(status_line_font) "-Adobe-Helvetica-Bold-R-Normal-*-*-120-*-*-*-*-*-*"
    set cbb(use_mems) 1
    set cbb(balloon,on) 1
    set cbb(web_browser) "netscape"

    # start time
    set cbb(sdate) ""
    set cbb(use_sdate) 0
    set cbb(report_sdate) ""
    set cbb(date_fmt) 1

    # account cache
    set cbb(cache) 0

    # use cryptography ?
    set cbb(use_crypt) 0
    set cbb(encrypt) ""
    set cbb(decrypt) ""

    # duplicate entries (0 = always insert, 1 = never insert, 2 = ask)
    # compare all fields except category
    set cbb(duplicate) 0
    set cbb(cmpfields) 0

    # Ok, so lets override them if we can ...
    if { [file exists $cbb(conf_file)] } {
	source $cbb(conf_file)
    }
    # why is 70 evil? - EAM
    if { $cbb(list_width) == 70 } {
	set cbb(list_width) 72
    }

    if { [file exists $cbb(save_cur_file)] } {
	source $cbb(save_cur_file)
    }

    # allow the user to put their own custom commands in ~/.wishrc
    if { [file exists "~/.wishrc"] } {
	source "~/.wishrc"
    }
}


# ----------------------------------------------------------------------------
# Parse command line arguments
# ----------------------------------------------------------------------------

proc parse_command_line {} {
    global cbb argc argv argv0 eng

    for { set i 0 } { $i < $argc } { incr i } {
	set arg [lindex $argv $i]
	
	if { $cbb(debug) } { puts "arg$i = $arg" }
	
	if { "$arg" == "--safe" } {
	    if { $cbb(debug) } { puts "setting safe mode" }
	    set cbb(safe_mode) 1
	    continue;
	}
	if { "$arg" == "--debug" } {
	    if { $cbb(debug) } { puts "forcing debug on" }
	    set cbb(debug) 1
	    # Set debugging flag on/off in perl engine
	    puts $eng "debug $cbb(debug)"; flush $eng
	    gets $eng result
	    if { $cbb(debug) } { puts $result }
	    continue;
	}
	if { "$arg" == "--devel" } {
	    if { $cbb(debug) } { puts "Including Developer options" }
	    set cbb(devel) 1
	    continue;
	}
	if { "$arg" == "--version" } {
	    puts "CBB $cbb(version)"
	    exit
	} 
	if { [string range $arg 0 0] == "-" } {
	    puts "Usage: [file tail $argv0] \[--safe\]\
		    \[--debug\] \[--devel\] \[--version\]"
	    exit
	}
	puts "Usage: [file tail $argv0] \[--safe\] \[--debug\] \[--devel\] \[--version\]"
    }
}
