#!/usr/bin/wish -f
#  'CBB' -- Check Book Balancer
#
#   main.tcl -- main window routines.
#
#  Written by Curtis Olson.  Started August 25, 1994.
#
#  Copyright (C) 1994 - 1999  Curtis L. Olson  - curt@me.umn.edu
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# main.tcl,v 0.11 2001/08/12 20:47:36 briank Exp


#------------------------------------------------------------------------------
# Setup main window
#------------------------------------------------------------------------------

proc setup_main {} {
    global cbb lib_path argv0

    # Setup main window parameters

    wm command . "[file tail $argv0]"
    wm group . .
    wm iconbitmap . @$lib_path/images/$cbb(icon_xbm)
    # specify absolute placement
    #wm geometry . +0+0
    # The following options will enable window resizing
    #wm minsize . 100 50
    #wm maxsize . 1000 700
    option add *font $cbb(default_font)

    # Setup container frames
    setup_containers

    # Setup menu bar
    option add *font $cbb(menu_font)
    setup_menubar .menubar
    setup_file_menu .menubar
    setup_edit_menu .menubar
    setup_functions_menu .menubar
    setup_extern_menu .menubar

    if { $cbb(devel) == 1 } { 
	setup_devel_menu .menubar
    }

    setup_help_menu .menubar
    setup_file_import_menu .menubar
    setup_file_export_menu .menubar
    setup_file_prefs_menu .menubar
    setup_prefs_crypt_menu .menubar
    setup_prefs_appear_menu .menubar
    setup_prefs_duplicate_menu .menubar
    setup_functions_goto_menu .menubar
    setup_functions_datefmt_menu .menubar

#   if { $cbb(devel) == 0 } {
#       tk_menuBar .menubar .menubar.file .menubar.edit .menubar.functions \
#		.menubar.extern .menubar.help
#   } else {
#	tk_menuBar .menubar .menubar.file .menubar.edit .menubar.functions \
#		.menubar.extern .menubar.devel .menubar.help
#   }
#
#    tk_bindForTraversal .

    # Setup headers
    setup_headers

    # Setup the transaction listbox and scrollbar
    setup_listbox 

    # Setup the entry area
    setup_entry_area

    # setup auto hiliting of entry fields if desired
    if { $cbb(auto_hilite) } {
	setup_auto_hilite
    }

    # Setup the command bar
    setup_command_bar

    # Setup the account list
    setup_acct_listbox

    # Setup the status line
    setup_status_line

    update
    wm deiconify .
}


#------------------------------------------------------------------------------
# Setup container frames
#------------------------------------------------------------------------------

proc setup_containers {} {
    frame .menubar -relief raised -borderwidth 2

    frame .head -relief raised -borderwidth 2
    frame .trans -relief raised -borderwidth 2
    frame .entry -relief raised -borderwidth 2
    frame .bar -borderwidth 2
    frame .acct -relief raised -borderwidth 2
    frame .status -relief raised -borderwidth 2
    pack .menubar -fill x 
    pack .head -fill x 
    pack .trans -fill both -expand 1
    pack .entry -fill both 
    pack .bar -fill x 
    pack .acct -fill x
    pack .status -fill x 
}


#------------------------------------------------------------------------------
# Setup headers
#------------------------------------------------------------------------------

proc setup_headers {} {
    global cbb

    label .head.line1 -font $cbb(fixed_header_font) \
            -text [format "%5s  %-10s  %-15s  %9s  %9s  %1s  %9s" \
            "Chk #" "Date" "Description" "Debit" "Credit" "" "Total"] \
            -padx 5 -pady -1 -foreground $cbb(head_color)
    label .head.line2 -font $cbb(fixed_header_font) \
            -text [format "%5s  %-10s  %-15s  %-9s" \
            "" "" "Comment" "Category"] -padx 4 -pady -1 \
	    -foreground $cbb(head_color)
    pack .head.line1 -side top -anchor w
    pack .head.line2 -side top -anchor w
}


#------------------------------------------------------------------------------
# Setup the transaction listbox and scrollbar
#------------------------------------------------------------------------------

proc setup_listbox {} {
    global cbb

    text .trans.list -width $cbb(list_width) -height $cbb(list_height) \
	    -spacing1 1 -takefocus 0 -exportselection false \
	    -font $cbb(fixed_font) -wrap none \
	    -state disabled \
	    -yscrollcommand ".trans.scroll set"

    if {[winfo depth .] > 1} {
	.trans.list tag configure color1 -background $cbb(list_line1_color) \
		-foreground $cbb(trans_text_color)
	.trans.list tag configure color2 -background $cbb(list_line2_color) \
		-foreground $cbb(trans_text_color)
	.trans.list tag configure negcolor1 -background $cbb(list_line1_color) \
		-foreground $cbb(trans_neg_color)
	.trans.list tag configure negcolor2 -background $cbb(list_line2_color) \
		-foreground $cbb(trans_neg_color)
	.trans.list tag configure futcolor1 -background $cbb(list_line1_color) \
		-foreground $cbb(future_color)
	.trans.list tag configure futcolor2 -background $cbb(list_line2_color) \
		-foreground $cbb(future_color)
	.trans.list tag configure futnegcolor1 -background \
		$cbb(list_line1_color) -foreground $cbb(future_neg_color)
	.trans.list tag configure futnegcolor2 -background \
		$cbb(list_line2_color) -foreground $cbb(future_neg_color)

	.trans.list tag configure cbbSel -background $cbb(hilite_color) \
		-foreground black
    } else {
	.trans.list tag configure color1
	.trans.list tag configure color2
    }

    bind .trans.list <ButtonRelease-1> {
	listHiliteTrans [.trans.list index @%x,%y]
    }

    bind .trans.list <Double-Button> {
	listHiliteTrans [.trans.list index @%x,%y]
	update_entry_area [.trans.list index @%x,%y]
    }

    scrollbar .trans.scroll -takefocus 0 -command ".trans.list yview" \
	    -relief flat
    pack .trans.scroll -side right -fill y 
    pack .trans.list -side left -fill both -expand 1
}


#------------------------------------------------------------------------------
# Setup the entry area
#------------------------------------------------------------------------------

proc setup_entry_area {} {
    global cbb lib_path

    set cbb(entry_area_focus) .entry.line1.date

    option add *font $cbb(fixed_font)

    image create photo "done" -file "$lib_path/images/mini-exclam.gif"
    image create photo "cancel" -file "$lib_path/images/mini-cross.gif"

    frame .entry.line1 
    frame .entry.line2
    pack .entry.line1 -side top -fill x -expand 1
    pack .entry.line2 -side top -fill x -expand 1

    entry .entry.line1.check  -width 5 -relief sunken -textvariable check
    entry .entry.line1.date   -width 10 -relief sunken -textvariable nicedate
    entry .entry.line1.desc   -width 15 -relief sunken -textvariable desc
    entry .entry.line1.debit  -width 9 -relief sunken -textvariable debit
    entry .entry.line1.credit -width 9 -relief sunken -textvariable credit
    entry .entry.line1.clear  -width 1 -relief sunken -textvariable cleared
    button .entry.line1.done  -image done -command done_entering -takefocus 0
    cbb_set_balloon .entry.line1.done "Done entering <Enter>"

    pack .entry.line1.check -side left
    pack .entry.line1.date -padx 8 -side left
    pack .entry.line1.desc -side left
    pack .entry.line1.debit -padx 8 -side left
    pack .entry.line1.credit -side left
    pack .entry.line1.clear -padx 8 -side left
    pack .entry.line1.done -side right

    label .entry.line2.space -width 17
    entry .entry.line2.com -width 15 -relief sunken -textvariable com
    entry .entry.line2.cat -width 9 -relief sunken -textvariable nicecat
    button .entry.line2.cancel -image cancel -command clear_entry_area -takefocus 0
    cbb_set_balloon .entry.line2.cancel "Cancel changes and start over <Meta-N>"

    pack .entry.line2.space -padx 5 -side left
    pack .entry.line2.com -side left
    pack .entry.line2.cat -padx 8 -side left
    pack .entry.line2.cancel -side right

    # setup some bindings

    # Change the bindtags so the following bindings execute first
    # the break makes sure any other default bindings are skipped.
    bindtags .entry.line1.check {.entry.line1.check Entry . all}

    # Don't use "all" bindings on these fields so we can control what happens 
    # with a <Tab> or <Shift-Tab>
    # bindtags .entry.line1.date {.entry.line1.date Entry .}
    bindtags .entry.line1.desc {.entry.line1.desc Entry .}
    bindtags .entry.line2.cat  {.entry.line2.cat Entry .}

    bind .entry.line1.check + {
	set check [inc_check $check]
	%W icursor end
	break
    }
    bind .entry.line1.check = {
	set check [inc_check $check]
	%W icursor end
	break
    }

    bind .entry.line1.check - {
	set check [dec_check $check]
	%W icursor end
	break
    }
    bind .entry.line1.check _ {
	set check [dec_check $check]
	%W icursor end
	break
    }

    bind .entry.line1.date + {
	set date [inc_date $date]
	set nicedate [date_to_nicedate $date]
	%W icursor end
	break
    }
    bind .entry.line1.date = {
	set date [inc_date $date]
	set nicedate [date_to_nicedate $date]
	%W icursor end
	break
    }

    bind .entry.line1.date - {
	set date [dec_date $date]
	set nicedate [date_to_nicedate $date]
	%W icursor end
	break
    }
    bind .entry.line1.date _ {
	set date [dec_date $date]
	set nicedate [date_to_nicedate $date]
	%W icursor end
	break
    }

    # setup custom math bindings for debit && credit fields
    if { [file exists "~/.cbbmath"] } {
        setup_custom_math "~/.cbbmath"
    }
}


#------------------------------------------------------------------------------
# Setup custom math bindings for debit and credit fields
#
# This contains some real ugliness ... I hope I never have to come back here
# to debug ... :-)
#------------------------------------------------------------------------------

proc setup_custom_math { filename } {
    global env

    set filehandle [open "$filename" r]
    while { [gets $filehandle line] >= 0 } {

	if { "[string range $line 0 0]" == "#" } {
	    # ignore this line
	} elseif { [string length $line] < 3  } {
	    # ignore this line
        } else {
	    set pieces [split $line "\t"]

	    set bindkey [lindex $pieces 0]
	    set units [lindex $pieces 1]
	    set expr [lindex $pieces 2]
	    set x 0

	    set bind_cmd "bind .entry.line1.debit $bindkey { \
		set x \$debit;
		set debit \[ expr $expr \];
                if { \[regexp \"\\\[(\\\]\\\[0-9\\\.\\\-\\\]+ $units\\\[)\\\]\" \$com\] } {
		    regsub \"\\\[(\\\]\\\[0-9\\\.\\\-\\\]+ $units\\\[)\\\]\" \$com \"(-\$x $units)\" com
		} else {
		    if { \"\$com\" == \"\" } {
			set com \"(-\$x $units)\";
		    } else {
			set com \"\$com (-\$x $units)\";
		    }
		}
		break
	    }"
	    eval $bind_cmd

	    set bind_cmd "bind .entry.line1.credit $bindkey { \
		set x \$credit;
		set credit \[ expr $expr \];
                if { \[regexp \"\\\[(\\\]\\\[0-9\\\.\\\-\\\]+ $units\\\[)\\\]\" \$com\] } {
		    regsub \"\\\[(\\\]\\\[0-9\\\.\\\-\\\]+ $units\\\[)\\\]\" \$com \"(\$x $units)\" com
		} else {
		    if { \"\$com\" == \"\" } {
			set com \"(\$x $units)\";
		    } else {
			set com \"\$com (\$x $units)\";
		    }
		}
		break
	    }"
	    eval $bind_cmd
	}
    }
    close $filehandle
}


#------------------------------------------------------------------------------
# Setup the command bar
#------------------------------------------------------------------------------

proc setup_command_bar { } {
    global cbb words

    option add *font $cbb(button_font)

    button .bar.new -text $words(button_new) -takefocus 0 \
	    -command { clear_entry_area }
    cbb_set_balloon .bar.new $words(button_new_balloon)

    button .bar.edit -text $words(button_edit) -takefocus 0 -command {
	if { [listGetCurTrans] >= 1 } {
	    update_entry_area [listGetCurTrans].0
	}
    }
    cbb_set_balloon .bar.edit $words(button_edit_balloon)

    button .bar.delete -text $words(button_delete) -takefocus 0 \
            -command {
	if { [listGetCurTrans] >= 1 } {
	    if { $cbb(debug) } { puts "Delete [listGetCurTrans].0" }
	    delete_trans [listGetCurTrans].0
	}
    } 
    cbb_set_balloon .bar.delete $words(button_delete_balloon)

    button .bar.splits -text $words(button_splits) -takefocus 0 -command { 
	cbbWindow.splits
	tkwait window .splits
    }
    cbb_set_balloon .bar.splits $words(button_splits_balloon)

    button .bar.balance -text $words(button_balance) -takefocus 0 \
	    -command { balance }
    cbb_set_balloon .bar.balance $words(button_balance_balloon)

    button .bar.save -text $words(button_save) -takefocus 0 -command {
	if { $cbb(cur_file) == "" } {
	    acctRename
	} else {
	    acctSave
	}
    }
    cbb_set_balloon .bar.save $words(button_save_balloon_clean)

    button .bar.quit -text $words(button_quit) -takefocus 0 -command { cbbQuit }
    cbb_set_balloon .bar.quit $words(button_quit_balloon)

    pack .bar.new .bar.edit .bar.delete .bar.splits .bar.balance .bar.save \
	.bar.quit -side left -fill x -expand 1 -padx 2 -pady 1
}

proc test_saved_state { } {
    global cbb yesno

    if { ! [acctIsDirty] } { return 0 }
    
    if { $cbb(auto_save) } {
	acctSave
	return 0
    } 
	
    cbbWindow.yesno "You have not saved your current changes.  \
	    Would you like to save before continuing?" yes
    tkwait window .yesno
    
    switch $yesno(result) {
	"yes" {
	    acctSave
	    return 0
	}
	"no" { return 0 }
	"cancel" { return 1 }
    }
    return 1
}


#------------------------------------------------------------------------------
# Setup the acct listbox and scrollbar
#------------------------------------------------------------------------------

proc setup_acct_listbox {} {
    global cbb yesno

    listbox .acct.list -width $cbb(list_width) -height $cbb(acctlist_height) \
	    -takefocus 0 -exportselection false \
	    -yscrollcommand ".acct.scroll set" -font $cbb(fixed_font)

    bind .acct.list <Double-Button> {
	if { "[.acct.list curselection]" == "" } { return }
	if { [test_saved_state] } { return }
	
	set line [.acct.list get [.acct.list curselection]]
	set name [string range $line 0 15]
	
	if { $cbb(debug) } { puts "load file: $name" }
	puts $eng "find_acct $name"; flush $eng;
	gets $eng result;

	if { "$result" == "none" } { 
	    cbbWindow.ok "can't find file for account"
	    tkwait window .ok
	    return
	}

	set pieces [split $result "\t"]
	set file  [lindex $pieces 0]
	if {$file == $cbb(cur_file)} { return }
	if { $cbb(debug) } { puts "setup_acct_listbox(): file=$file" }
	
	set result [load_account $cbb(cur_group) $cbb(cur_path)/$cbb(cur_group)/$file]
	if {"$result" == "error"} { 
	    if {$cbb(cur_file) == ""} {return}
	    set result [load_account $cbb(cur_group) $cbb(cur_path)/$cbb(cur_group)/$cbb(cur_file)];
	    if {"$result" == "error"} { 
		cbbWindow.ok "can't reload original account $cbb(cur_path)/$cbb(cur_file)"
		tkwait window .ok
		return
	    }
	}
    }

    pack .acct.list -side left -fill both -expand 1

    scrollbar .acct.scroll -takefocus 0 -command ".acct.list yview" \
	    -relief flat
    pack .acct.scroll -side right -fill y -expand 1
}


#------------------------------------------------------------------------------
# Setup the status line
#------------------------------------------------------------------------------

proc setup_status_line { } {
    global cbb argv0

    label .status.line -text "Welcome to [file tail $argv0]" -font $cbb(status_line_font)
    frame .status.files

    pack .status.files -fill both -expand 1
    pack .status.line -fill both -expand 1

    label .status.files.acct -text "account: " -font $cbb(status_line_font)
    label .status.files.group -text "group: " -font $cbb(status_line_font)

    pack .status.files.acct -side left 
    pack .status.files.group -side left
}


#------------------------------------------------------------------------------
# Functions for the text box
#------------------------------------------------------------------------------

# $date_now should be the calendar on the wall date, nothing to do with the
# date of the current transaction.
# listing from "all_trans" returns file two lines at a time,
# these come in as line1 & line2.  the format is position dependent. 
# line1:
#(check)(date)      (description)   (debit)     (credit)   c   (total)     (key) 
#        19991217   Kitty                 0.00     100.00  *    8829.39    19991217-01
# line2:
#(unused          ) (comment)        (category)                            (key)
#                   deposit interes  interest                              19991231-00
#0         1         2         3         4         5         6         7 
#012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789

proc listAddTrans {line1 line2} {
    global cbb date start_pos

    if { $cbb(debug) } { puts " --> $line1" }
    set total [string range $line1 60 69]
    if { $cbb(debug) } { puts " --> $total" }
    set negative 0
    if { $total < 0 } { set negative 1 }

    set p1   [string range $line1 0 6]
    set date [string range $line1 8 15]
    set p2   [string range $line1 17 84]
    set nicedate [date_to_nicedate $date]
    set line1 [format "%s%s%s" $p1 $nicedate $p2]

    # tmp_date is the year part of the key (yyyymmdd)
    if ![info exists cbb(date_now)] { set cbb(date_now) [clock format [clock seconds] -format "%Y%m%d"] }
    set tmp_date [string range $line1 74 81]
    if { $cbb(debug) } { puts "$cbb(date_now) - $tmp_date" }

    # Future transactions in paler color, deficits in redder color
    if { $negative } {
	if { $tmp_date > $cbb(date_now) } {
	    .trans.list insert end "$line1\n" futnegcolor1
	    .trans.list insert end "$line2\n" futnegcolor2
	} else {
	    .trans.list insert end "$line1\n" negcolor1
	    .trans.list insert end "$line2\n" negcolor2
	}
    } else {
	if { $tmp_date > $cbb(date_now) } {
	    .trans.list insert end "$line1\n" futcolor1
	    .trans.list insert end "$line2\n" futcolor2
	} else {
	    .trans.list insert end "$line1\n" color1
	    .trans.list insert end "$line2\n" color2
	}
    }

    if { $tmp_date <= $date } {
	set start_pos [listGetSize]
    }
}


proc listGetSize {} {
    scan [.trans.list index end] "%d.%d" line col
    return [expr $line - 2]
}


proc listGetCurTrans {} {
    global cbb

    if { "[.trans.list tag nextrange cbbSel 1.0 end ]" != "" } {
	scan [.trans.list tag nextrange cbbSel 1.0 end ] "%d.%d" line col
	# set line [expr $line + 1]
    } else {
	set line -1
    }

    if { $cbb(debug) } { puts $line }

    return $line
}


proc listHiliteTrans index {
    global cbb

    scan $index "%d.%d" line col


    .trans.list tag remove sel 1.0 end
    .trans.list tag remove cbbSel 1.0 end

    if { $line < 1 } {
	set line 1
    }

    if { [ expr $line / 2.0 ] != [ expr $line / 2] } {
	set index1 $line
	set index2 [ expr $line + 1 ]
    } else {
	set index1 [ expr $line - 1 ]
	set index2 $line 
    }
	
    .trans.list tag add cbbSel ${index1}.0 ${index2}.0lineend
    .trans.list see ${index1}.0
    .trans.list see ${index2}.0
}	


#------------------------------------------------------------------------------
# Functions for entry area
#------------------------------------------------------------------------------

proc update_globals result {
    global cbb eng key date nicedate check desc debit credit cat
    global nicecat com cleared total 

    # clear entries
    set date ""; set check ""
    set desc ""; set debit 0.00; set credit 0.00; set cat ""; set nicecat ""
    set com ""; set cleared ""; set total 0.00

    set pieces [split $result "\t"]
    set key [lindex $pieces 0]
    set date [lindex $pieces 1]
    set nicedate [date_to_nicedate $date]

    set check [lindex $pieces 2]
    set desc [lindex $pieces 3]
    set cbb(mem_match) $desc
    scan [lindex $pieces 4] "%f" debit
    scan [lindex $pieces 5] "%f" credit
    set debit [format "%.2f" $debit]; 
    set credit [format "%.2f" $credit]; 
    set cat [lindex $pieces 6]
    set nicecat [cat_to_nicecat $cat]
    set com [lindex $pieces 7]
    set cleared [lindex $pieces 8]
    scan [lindex $pieces 9] "%f" total
}


# given a memorized transaction, update the relevant fields
proc update_from_mem result {
    global eng desc debit credit cat nicecat com cbb

    set pieces [split $result "\t"]
    set desc [lindex $pieces 3]

    # only update the unaltered components
    if { [expr {$debit == 0.0} && {$credit == 0.0} ] } {
	set debit  [format "%.2f" [lindex $pieces 4]]
	set credit [format "%.2f" [lindex $pieces 5]]
    }
    if {! [string compare $cat ""] } {
	set cat [lindex $pieces 6]
	set nicecat [cat_to_nicecat $cat]
    }
    if {! [string compare $com ""] } {
	set com [lindex $pieces 7]
    }


}

proc find_index_from_key args {
    global cbb
    # given a newkey, return the index of the first affected transaction

    set arglist [split $args]
    set cbb(index1) [lindex $arglist 0]
    set newkey [lindex $arglist 1]

    if { $cbb(debug) } { puts "find: cbb(index1) = $cbb(index1)  newkey = $newkey" }

    if { [expr $cbb(index1) / 2.0] == [expr $cbb(index1) / 2] } {
	set cbb(index1) [expr $cbb(index1) - 1]
    }

    if { $cbb(index1) < 1 } {
	set cbb(index1) 1
    }

    set line [.trans.list get $cbb(index1).0 $cbb(index1).0lineend ]
    set key [string range $line 74 end]

    if { $cbb(debug) } { puts "target = $newkey  current = $key" }

    # this is not a very fast search.  we should bisect from the start of the list.
    if { [string compare "$newkey" "$key"] == -1 } {
	# we changed the date to something previous
	while { [expr [string compare "$newkey" "$key"] == -1 && $cbb(index1) > 0]} {
	    set cbb(index1) [expr $cbb(index1) - 2]
	    set line [.trans.list get $cbb(index1).0 $cbb(index1).0lineend ]
            set key [string range $line 74 end]
	    if { $cbb(debug) } { puts "target = $newkey  current = $key" }
	}
	return [expr $cbb(index1)]
    } else {
	# we changed the date to something forward or this is the trivial case
	return [expr $cbb(index1) - 2]
    }
}


proc find_index_from_date date {
    global cbb
    # given a date in the form yyyymmdd, return the index of the transaction
    # which is previous to the next higher date

    if { $cbb(debug) } { puts "find: date = $date" }

    set index [listGetSize]

    set line [.trans.list get ${index}.0 ${index}.0lineend ]
    set linedate [string range $line 74 81]

    if { $cbb(debug) } { puts "target = $date  current = $linedate" }

    # not a very fast search, should bisect back to start
    while { [expr [string compare "$date" "$linedate"] == -1 && $index > 0]} {
	set index [expr $index - 2]
	set line [.trans.list get ${index}.0 ${index}.0lineend ]
	set linedate [string range $line 74 81]
	if { $cbb(debug) } { puts "target = $date  current = $linedate" }
    }

    return [expr $index]
}


proc update_rest args {
    global cbb key eng date nicedate check desc debit credit cat
    global nicecat com cleared total

    set arglist [split $args]
    set cbb(index1) [lindex $arglist 0]
    set newkey [lindex $arglist 1]

    if { $cbb(debug) } { puts "update_rest: $cbb(index1) $newkey" }

    # save the current listbox view ...

    set yview_list [.trans.list yview]
    if { $cbb(debug) } { puts "Saving current view: $yview_list" }
    set yview_saved [lindex $yview_list 0]
    if { $cbb(debug) } { puts $yview_saved }

    # delete everything from the change forward, then rebuild our list from 
    # there

    set cbb(index1) [find_index_from_key $cbb(index1) $newkey]
    if { [expr $cbb(index1) < 0] } {
        set cbb(index1) 1
    }
    set cbb(index2) [expr $cbb(index1) + 1]

    if { $cbb(debug) } { puts "deleting from:  $cbb(index1).0 to end" }

    set line [.trans.list get $cbb(index1).0 $cbb(index1).0lineend ]
    set key [string range $line 74 end]
    .trans.list configure -state normal
    .trans.list delete $cbb(index1).0 end
    .trans.list configure -state disabled
 
    if { $cbb(debug) } { puts [string range $line 74 end] }
    if { $cbb(debug) } { puts "adding entries from $key to end" }

    if { $cbb(index1) == 1 } {
    	puts $eng "first_trans"; flush $eng
    } else {
        puts $eng "find_trans $key"; flush $eng
	.trans.list configure -state normal
	.trans.list insert end "\n"
	.trans.list configure -state disabled
    }
    .trans.list configure -state normal
    gets $eng result
    while { $result != "none" } {
        update_globals $result

        set checklen [string length $check]
        if { $checklen > 5 } {
	    set cutcheck [string range $check [expr $checklen - 5] end]
        } else {
	    set cutcheck $check
        }
        set cutdesc [string range $desc 0 14]
        set cutcom [string range $com 0 14]

	listAddTrans \
		[format "%5s   %-8s   %-15s  %9.2f  %9.2f  %-1s  %9.2f %14s" \
		$cutcheck $date $cutdesc $debit $credit $cleared $total \
		$key] \
		[format "%5s  %-10s  %-15s  %-9s %39s" "" "" $cutcom $nicecat \
		$key] 

        # try keep the selection with the original transaction
        if { $key == $newkey } {
            set cbb(selected) [expr [listGetSize] - 1]
	    listHiliteTrans $cbb(selected).0
	    set tmp_date $cbb(cur_date)
    	    set cbb(cur_date) [nicedate_to_date $nicedate $tmp_date]
 
	    if { "$check" != "" } { set cbb(next_chk) $check }
         }

        puts $eng "next_trans"; flush $eng
        gets $eng result
    }
    .trans.list configure -state disabled

    # now try to restore the current view ...

    .trans.list yview moveto $yview_saved
    set temp [listGetCurTrans]
    if { $temp >= 1 } {
	.trans.list see ${temp}.0
    } else {
	.trans.list see 1.0
    }

    # Update balance window
    updateBalanceWindow
}


proc update_line args {
    global cbb key eng date nicedate check desc debit credit cat
    global nicecat com cleared total

    set arglist [split $args]
    set cbb(index1) [lindex $arglist 0]
    set key [lindex $arglist 1]
    set temp [listGetCurTrans]
    if { $temp >= 1 } {
	set cbb(selected) $temp
    } else {
	set cbb(selected) 1
    }

    if { $cbb(debug) } { puts "update_line: $cbb(index1) $key" }

    # delete trans and re-insert

    set cbb(index2) [expr $cbb(index1) + 1]

    if { $cbb(debug) } { puts "deleting from:  $cbb(index1) to $cbb(index2)" }

    set line [.trans.list get $cbb(index1).0 $cbb(index1).0lineend ]
    set key [string range $line 74 end]
    .trans.list configure -state normal
    .trans.list delete $cbb(index1).0 [expr $cbb(index2) + 1].0
    .trans.list configure -state disabled
 
    if { $cbb(debug) } { puts "re-inserting entry" }

    puts $eng "find_trans $key"; flush $eng
    gets $eng result

    update_globals $result

    set checklen [string length $check]
    if { $checklen > 5 } {
	set cutcheck [string range $check [expr $checklen - 5] end]
    } else {
	set cutcheck $check
    }
    set cutdesc [string range $desc 0 14]
    set cutcom [string range $com 0 14]

    .trans.list configure -state normal
    .trans.list insert $cbb(index1).0 \
           [format "%5s  %-10s  %-15s  %9.2f  %9.2f  %-1s  %9.2f %14s\n" \
           $cutcheck $nicedate $cutdesc $debit $credit $cleared $total \
	   $key] color1
    .trans.list insert $cbb(index2).0 \
           [format "%5s  %-10s  %-15s  %-9s %39s\n" "" "" $cutcom $nicecat \
	   $key] color2
    .trans.list configure -state disabled

    listHiliteTrans $cbb(selected)
}


proc clear_entry_area {} {
    global cbb key eng date nicedate check desc debit credit cat
    global nicecat com cleared total

    set key ""; set date ""; set check ""; 
    set desc ""; set debit 0.00; set credit 0.00; set cat ""
    set nicecat ""; set com ""; set cleared ""; set total 0.00

    # converted calls to perl to TCL code for dates
    if { "$cbb(cur_date)" == "" } { 
	set cbb(cur_date) [clock format [clock seconds] -format "%Y%m%d"]
    }
    set date $cbb(cur_date)
    set nicedate [date_to_nicedate $cbb(cur_date)]

    # get internal sdate, if sdate is defined
    if { $cbb(sdate) != "" } {
	puts $eng "start_date $cbb(sdate)"; flush $eng
	gets $eng cbb(int_sdate)
    }

    if { $cbb(debug) } { puts $nicedate; puts $date }

    reset_desc_memory

    focus $cbb(entry_area_focus)
}


proc update_entry_area lineindex {
    global cbb key eng cat
    global nicecat com cleared total

    scan $lineindex "%d.%d" item col
    set item [expr $item + 1]

    reset_desc_memory
    .status.line configure -text ""

    set cbb(selected) $item

    if { [expr $item % 2] } {
	set cbb(index1) [expr $item - 1]
	set cbb(index2) $item
    } else {
	set cbb(index1) $item
	set cbb(index2) [expr $item + 1]
    }

    set line [.trans.list get $cbb(index1).0 $cbb(index1).0lineend ] 
    set key [string range $line 74 end]
    if { $cbb(debug) } { puts "looking for $key" }

    puts $eng "find_trans $key"; flush $eng
    gets $eng result
    if { $cbb(debug) } { puts $result }

    if { $result != "none" } {
    	update_globals $result
    }

    # warn if about to edit a closed transaction
    if { "$cleared" == "x" || "$cleared" == "X" } {
	.status.line configure -text "warning: 'closed' transaction" -foreground red
    }

    # warn if about to edit a transfer transaction
    if { "[string range $cat 0 0]" == "\[" } {
	# clear this line when clicking on a new field
	.status.line configure -text "warning: 'transfer' transaction" -foreground red
    }


    focus $cbb(entry_area_focus)
}


proc done_entering {} {
    global cbb yesno key eng date nicedate check desc debit 
    global credit cat nicecat com cleared total addcat

    if { $cbb(debug) } { puts "Done entering ..." }

    if { $check != "" } {
	set entry_area_focus .entry.line1.check
    } else {
	set entry_area_focus .entry.line1.date
    }

    # check for a valid file
    if {"$cbb(cur_file)" == ""} {
	cbbWindow.ok "You must Make or Load an Account First."
	tkwait window .ok
	return
    } 

    acctSetDirty

    if {"$nicedate" == ""} { set nicedate [date_to_nicedate $cbb(cur_date)] }
    if {"$debit" == ""} { set debit 0.0 }
    if {"$credit" == ""} { set credit 0.0 }

    # do some consistency checking here
    if { "$desc" == "" } {
	cbbWindow.yesno "You have not entered anything in the description field.  Is this OK?" yes
	tkwait window .yesno
	if { "$yesno(result)" != "yes" } { 
          set cbb(entry_area_focus) .entry.line1.desc
          focus $cbb(entry_area_focus)
          return 
        }
    }

    # here we convert the input value to nicedate 
    # input may have arbitrary separators and missing month,year - EAM
    set date [nicedate_to_date $nicedate $cbb(cur_date)]
    set nicedate [date_to_nicedate $date]

    # check validity of $cat
    if { "$nicecat" != "-Splits-" } { set cat $nicecat }

    # If the user didn't enter a category, they have two choices:  Select no and
    # return to the transaction (possibly for another chance to enter a category),
    # or select yes and continue with processing.  This is the original
    # functionality in 0.8.2.
    if { "$cat" == "" } { 
	cbbWindow.yesno "You have not entered anything in the category field.  Is this OK?" yes
	tkwait window .yesno
	if { "$yesno(result)" != "yes" } { 
          set cbb(entry_area_focus) .entry.line2.cat
          focus $cbb(entry_area_focus)
          return 
        }
    } elseif { "$nicecat" != "-Splits-" } {
	set cbb(cat_entry) 1
	set cbb(cat_found) 0
	set cbb(cat_partial) $cat
	puts $eng "find_cat 0\t$cat"; flush $eng
	gets $eng result
	gets $eng match 

	set pieces [split $result "\t"]
	set stat  [lindex $pieces 0]
	set pieces [split $match "\t"]
	set match [lindex $pieces 1]

	if {![string compare $stat "PARTIAL"]} {
	    cbbWindow.yesno "Category partially matches [string range $result 8 end].  Use category $cat?" yes
	    tkwait window .yesno
	    if { "$yesno(result)" != "yes" } { return }
	    set match $cat
	} 
	if {![string compare $stat "NONE"]} {
	    set addcat(cat) $cat
	    set addcat(mode) "missing"
	    cbbWindow.newcat
	    tkwait window .newcat
	    if { $cbb(debug) } { puts $addcat(result) }
	    if { "$addcat(result)" != "yes" } { return }
	    set match $cat
	} 
	set nicecat $match
	set cat $match
    }

    # verify cleared field
    set cleared [string range $cleared 0 0]
    switch $cleared {
	"x" { }
	"X" { }
	"*" { }
	"?" { }
	""  { }
	default { set cleared "" }
    }

    # transfer transaction: the transfer is added to the transaction list with debit and credit 
    # reversed.  these are found and applied to the appropriate accounts on file save, 
    # which keeps transactions consistent before and after saves
    # THERE is a problem with transfers which are split.  first, the transfer is not detected
    # second, we would need to write a subset of the transaction to the target account
    # third, and worst of all, the process which deletes transfers would not be able to find the 
    # correct match between the to and from accounts.

    # new entry -- insert it in the list
    if { $cbb(debug) } { puts "key: $key" }
    if { "$key" == "" } {
	if { "[string range $cat 0 0]" == "\[" } { 	    
	    puts $eng "create_xfer $date\t$check\t$desc\t$credit\t$debit\t$cat\t$com\t$cleared\t0.00"
	    flush $eng
	    gets $eng result
	} 

	puts $eng "create_trans $date\t$check\t$desc\t$debit\t$credit\t$cat\t$com\t$cleared\t0.00"
	flush $eng
        gets $eng result
        if { $cbb(debug) } { puts "result:  create_trans $result" }
        if { "$result" == "error" } {
	    cbbWindow.ok "error creating transaction" 
	    tkwait window .ok
        } 
	undoRegister "insert $result"
	update_rest [listGetSize] [string range $result 0 10]

	# try keep the entry area in sync with the selection
	goto $cbb(selected)
	set cbb(entry_area_focus) $entry_area_focus
	clear_entry_area
	return
    }

    # check validity of the key
    if { [expr {[string length "$key"] != 11} || {[string first "-" "$key"] != 8}]} {
	cbbWindow.ok "Bad key value '$key'.  This transaction is aborted."
	tkwait window .ok
	set key ""
	return
    }
    
    # existing entry -- alter it in the list
    # first record the official version of this transaction so we can be
    # able to undelete it later
    puts $eng "find_trans $key"; flush $eng
    gets $eng origresult

    # transfer transaction - the transfer is added to the transaction list with debit and credit 
    # reversed and the $cleared flag set to t (existing xfer).  these are found and applied to the appropriate 
    # accounts on file save, which keeps transactions consistent before and after saves
    if { "[string range $cat 0 0]" == "\[" } { 	    
	set pieces [split $origresult "\t"]
	set odate [lindex $pieces 1]
	set ocheck [lindex $pieces 2]
	set odesc [lindex $pieces 3]
	set odebit [lindex $pieces 4]
	set ocredit [lindex $pieces 5]
	set ocat [lindex $pieces 6]
	set ocom [lindex $pieces 7]
	set oclear [lindex $pieces 8]
	puts $eng "delete_xfer $odate\t$ocheck\t$odesc\t$ocredit\t$odebit\t$ocat\t$ocom\t$oclear\t0.00"
	flush $eng
	gets $eng result
	puts $eng "create_xfer $date\t$check\t$desc\t$credit\t$debit\t$cat\t$com\t$cleared\t0.00"
	flush $eng
	gets $eng result
    } 
    
    # update transaction
    puts $eng "update_trans $key\t$date\t$check\t$desc\t$debit\t$credit\t$cat\t$com\t$cleared\t0.00"
    flush $eng
    gets $eng result
    if { $cbb(debug) } { puts "result:  create_trans $result" }
    if { "$result" == "error" } {
	cbbWindow.ok "error editing transaction" 
	tkwait window .ok
    } 
    if { "$cbb(index1)" == "" } { set cbb(index1) [listGetSize] }
    update_rest $cbb(index1) [string range $result 0 10]
    undoRegister "edit [string range $result 0 10]\t$origresult"

    # try keep the entry area in sync with the selection
    goto $cbb(selected)
    set cbb(entry_area_focus) $entry_area_focus
    clear_entry_area
}


proc delete_trans lineindex {
    global cbb yesno eng cat cleared

    scan $lineindex "%d.%d" item col
    # set item [expr $item - 1]

    acctSetDirty

    if { [expr $item / 2.0] != [expr $item / 2] } {
	set cbb(index1) $item
	set cbb(index2) [expr $item + 1]
    } else {
	set cbb(index1) [expr $item - 1]
	set cbb(index2) $item
    }

    set line [.trans.list get $cbb(index1).0 $cbb(index1).0lineend ]
    set key [string range $line 74 end]

    # first record the official version of this transaction so we can be
    # able to undelete it later
    puts $eng "find_trans $key"; flush $eng
    gets $eng result
    update_globals $result
    undoRegister "delete $result"

    if { "[string range $cat 0 0]" == "\[" } {
	cbbWindow.ok "Notice:  You are deleting a ``Transfer'' transaction.  \
		The corresponding transaction in the file ``$cat'' will \
		also be deleted."
	tkwait window .ok
    }

    if { "$cleared" == "x" || "$cleared" == "X" } {
	cbbWindow.yesno "You are deleting a ``Closed'' transaction.  Continue \
		with delete?" no
	tkwait window .yesno
	
	if { "$yesno(result)" == "yes" } {
	} elseif { "$yesno(result)" == "no" } {
	    return
	} elseif { "$yesno(result)" == "cancel" } {
	    return
	}
    }

    if { "[string range $cat 0 0]" == "\[" } {
	set pieces [split $result "\t"]
	set odate [lindex $pieces 1]
	set ocheck [lindex $pieces 2]
	set odesc [lindex $pieces 3]
	set odebit [lindex $pieces 4]
	set ocredit [lindex $pieces 5]
	set ocat [lindex $pieces 6]
	set ocom [lindex $pieces 7]
	set oclear [lindex $pieces 8]
	puts $eng "delete_xfer $odate\t$ocheck\t$odesc\t$ocredit\t$odebit\t$ocat\t$ocom\t$oclear\t0.00"
	flush $eng
	gets $eng result
    } 
    puts $eng "delete_trans $key"; flush $eng
    gets $eng result

    if { $cbb(debug) } { puts "deleting:  $result" }

    update_rest $cbb(index1) $key

    goto $cbb(index1)

    clear_entry_area
}


#------------------------------------------------------------------------------
# Miscellaneous functions
#------------------------------------------------------------------------------

proc goto line {
    global cbb

    if { $cbb(debug) } { puts "Size = [listGetSize]  Goto = $line" }

    set max [expr [listGetSize] - 1]
    if { $line > $max } {
	set line $max
    }

    if { [expr $line / 2.0] == [expr $line / 2] } {
        set line [expr $line - 1]
    }

### .trans.list see $line
### .trans.list see [expr $line + 1]
### .trans.list selection clear 0 end
### .trans.list selection set $line $line
    listHiliteTrans $line
}


proc acctSetClean { } {
    global cbb words

    set cbb(clean) 1

    # change appearance of "save" button (save -> save!)
    if { [winfo exists .bar.save] } {
	.bar.save configure -text $words(button_save)
	cbb_set_balloon .bar.save $words(button_save_balloon_clean)
    }

    # we need to recalc balance and redisplay in the list here

}


proc acctSetDirty { } {
    global cbb words
    
    set cbb(clean) 0
    if { [winfo exists .bar.save] } {
	.bar.save configure -text $words(button_save_excl)
	cbb_set_balloon .bar.save $words(button_save_balloon_dirty)
    }
}


proc acctIsDirty { } {
    global cbb

    return [expr 1 - $cbb(clean)]
}

proc cat_to_nicecat { cat } {
    
    if { [string range $cat 0 0] == "|" } {
        set nicecat "-Splits-"
    } else {
    	set nicecat $cat
    }
    set nicecat [string range $nicecat 0 8]

    return $nicecat
}

proc setMainInfo {} {
    global argv0 cbb

    if {"$cbb(cur_name)" == ""} {
	wm title . "[file tail $argv0]"
	wm iconname . "[file tail $argv0]"
    } else {
	wm title . "[file tail $argv0] - $cbb(cur_name)"
	wm iconname . "[file tail $argv0] - $cbb(cur_name)"
    }
    .status.files.acct configure -text "account: $cbb(cur_name)"
    .status.files.group configure -text "group: $cbb(cur_group)"
}
