#!/usr/bin/wish -f
#  'CBB' -- Check Book Balancer
#
#   splits.tcl -- Routines to handle splits.
#
#  Written by Curtis Olson.  Started December 7, 1996.
#
#  Copyright (C) 1994 - 1999  Curtis L. Olson  - curt@me.umn.edu
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# splits.tcl,v 0.10 2001/07/13 04:05:59 briank Exp


#------------------------------------------------------------------------------
# Procedures for category split processing
#------------------------------------------------------------------------------

proc cbbWindow.splits {} {
    global cbb desc cat credit debit cats coms amts 

    if {[winfo exists .splits] == 1} { destroy .splits }
    # ; doneSplits; return 

    # set up initial values for window
    if { "$credit" == "" } { set credit 0.00 }
    if { "$debit" == "" } { set debit 0.00 }
    if { $cbb(debug) } { puts $cat }

    # define window & basic contents
    option add *font $cbb(dialog_font)
    toplevel .splits
    wm title .splits "Category Splits"
    wm iconname .splits "Category Splits"
    frame .splits.frame -borderwidth 2 -relief raised
    pack .splits.frame -side top -fill both -expand 1

    label .splits.frame.head -text "[string range $desc 0 30] =:= [format "%.2f" [expr $credit - $debit]]"
    pack .splits.frame.head -anchor w -fill x

    option add *font $cbb(fixed_header_font)
    frame .splits.frame.label
    label .splits.frame.label.cat -text "  Category     "
    label .splits.frame.label.com -text "   Comment     "
    label .splits.frame.label.amt -text " Amount  "
    pack .splits.frame.label -side top -fill x
    pack .splits.frame.label.cat .splits.frame.label.com .splits.frame.label.amt -side left -expand 1

    set pieces [split $cat |]
    option add *font $cbb(fixed_font)
    for { set i 0 } { $i < $cbb(max_splits) } { incr i } {
	# setup widgets
	frame .splits.frame.line$i

	entry .splits.frame.line$i.cat$i -relief sunken -width 15 -textvariable cats($i)
	entry .splits.frame.line$i.com$i -relief sunken -width 15 -textvariable coms($i)
	entry .splits.frame.line$i.amt$i -relief sunken -width 9 -textvariable amts($i)

	# why is this like this?
	set command "bindtags .splits.frame.line$i.cat$i {.splits.frame.line$i.cat$i Entry .}"
	eval $command

	bind .splits.frame.line$i.cat$i <FocusIn> { reset_split_memory }
	bind .splits.frame.line$i.cat$i <Tab> { splitsLeavingCat %W }
	bind .splits.frame.line$i.cat$i <Control-Tab> { splitsMatchCat %W }

	bind .splits.frame.line$i.amt$i <FocusOut> { splitsLeavingAmt %W }

        bind .splits.frame.line$i.cat$i <Return> { doneSplits }
        bind .splits.frame.line$i.com$i <Return> { doneSplits }
        bind .splits.frame.line$i.amt$i <Return> { doneSplits }

        bind .splits.frame.line$i.cat$i <KP_Enter> { doneSplits }
        bind .splits.frame.line$i.com$i <KP_Enter> { doneSplits }
        bind .splits.frame.line$i.amt$i <KP_Enter> { doneSplits }

	bind .splits.frame.line$i.cat$i <Shift-Tab>    { focus [tk_focusPrev %W] }
	bind .splits.frame.line$i.com$i <Shift-Tab>    { focus [tk_focusPrev %W] }
	bind .splits.frame.line$i.amt$i <Shift-Tab>    { focus [tk_focusPrev %W] }

	bind .splits.frame.line$i.cat$i <ISO_Left_Tab> { focus [tk_focusPrev %W] }
	bind .splits.frame.line$i.com$i <ISO_Left_Tab> { focus [tk_focusPrev %W] }
	bind .splits.frame.line$i.amt$i <ISO_Left_Tab> { focus [tk_focusPrev %W] }

	bind .splits.frame.line$i.cat$i <Meta-c> { cbbWindow.catwin }
	bind .splits.frame.line$i.com$i <Meta-c> { cbbWindow.catwin }
	bind .splits.frame.line$i.amt$i <Meta-c> { cbbWindow.catwin }

	bind .splits.frame.line$i.cat$i <Alt-c> { cbbWindow.catwin }
	bind .splits.frame.line$i.com$i <Alt-c> { cbbWindow.catwin }
	bind .splits.frame.line$i.amt$i <Alt-c> { cbbWindow.catwin }

	set cats($i) [lindex $pieces [expr 1 + $i * 3]]
	set coms($i) [lindex $pieces [expr 2 + $i * 3]]
	set amts($i) [lindex $pieces [expr 3 + $i * 3]]
	pack .splits.frame.line$i -side top -fill x -expand 1
	pack .splits.frame.line$i.cat$i .splits.frame.line$i.com$i .splits.frame.line$i.amt$i -side left -expand 1 -fill x
    }

    # try to do some fancier initial splits handling
    if { "[string range $cat 0 0]" != "|" } {
	if { $cbb(debug) } { puts "not a split = $cat" }
	set cats(0) $cat
	set amts(0) [format "%.2f" [expr $credit - $debit]]
    }
    
    # create lines @ bottom -- must be created after amts, cats, coms are defined above
    label .splits.frame.total -borderwidth 2 -text "Sum = [splitsSum]" -font $cbb(dialog_font)
    pack .splits.frame.total -side bottom -after .splits.frame.label -fill x -expand 1

    button .splits.frame.dismiss -text " Dismiss " -font $cbb(button_font) -takefocus 0 -command { doneSplits }
    pack .splits.frame.dismiss -side bottom -after .splits.frame.label -fill x -padx 8 -pady 4

    focus .splits.frame.line0.cat0
    update
}

proc reset_split_memory { } {
    global cbb

    set cbb(splt_entry) 0
    set cbb(splt_found) 0
    set cbb(splt_match) ""
    set cbb(splt_partial) ""
}
    
proc splitsMatchCat { field } {
    global cbb eng cats addcat

    set pos [string last cat $field]
    set N [string range $field [expr $pos + 3] end]
    if { $cbb(debug) } { puts "$cats($N)" }

    if {! [string length $cbb(splt_partial)] } { 
	if {! [string length $cats($N)] } { return }
	set cbb(splt_partial) $cats($N)
    }
    
    puts $eng "find_cat $cbb(splt_entry)\t$cbb(splt_partial)"; flush $eng
    gets $eng result
    gets $eng match 
    if { $cbb(debug) } { puts $result; puts $match}
    
    set pieces [split $match "\t"]
    set cats($N) [lindex $pieces 1]
    
    set pieces [split $result "\t"]
    set stat  [lindex $pieces 0]
    
    incr cbb(splt_entry)
    set cbb(splt_found) 1
    
    # nothing special for responses PARTIAL, COMPLET, and UNIQUE
    if {"$stat" == "NONE" } {
	set cats($N) $cbb(splt_partial)
	set cbb(splt_entry) 0
	set cbb(splt_found) 0
    }
    tkEntrySetCursor $field end
    set cbb(splt_match) $cats($N)
}


proc doneSplits { } {
    global cbb cat nicecat credit debit cats coms amts 

    # check to see if splits add up
    if { $debit > 0 } {
	if { [splitsSum] != [format "%.2f" -$debit] } {
	    cbbWindow.ok "Warning:  Sum of splits, [splitsSum], does not \
		    match transaction value, -$debit."
	    tkwait window .ok
	}
    } else {
	if { [splitsSum] != [format "%.2f" $credit] } {
	    cbbWindow.ok "Warning:  Sum of splits, [splitsSum], does not \
		    match transaction value, $credit."
	    tkwait window .ok
	}
    }

    # recreate internal splits category record
    set i 0
    set Ns 0
    set cat |
    while { $i < $cbb(max_splits) } {
	if { "$amts($i)" != "" } {
	    append cat $cats($i) | $coms($i) | [format "%.2f" $amts($i)] |
	    incr Ns
	    set save $i
	}
	incr i
    }
    if { $Ns == 1 } {
	set cat $cats($save)
	set nicecat $cat
    }
    if { $Ns == 0 } {
	set cat ""
	set nicecat ""
    }
    if { $Ns > 1 } {
	set nicecat "-Splits-"
    }
    destroy .splits
}

proc splitsLeavingCat { field } {
    global cbb eng cats addcat

    # just left a split category field

    set pos [string last cat $field]
    set N [string range $field [expr $pos + 3] end]
    if { $cbb(debug) } { puts "$cats($N)" }

    # skip past on these conditions
    if {! [expr [string length $cats($N)]] } { focus [tk_focusNext $field]; reset_split_memory; return }
    if {! [string compare $cats($N) $cbb(splt_match)] } { 
	if { !$cbb(splt_found) } {
	    set addcat(cat) $cats($N)
	    set addcat(mode) "missing"
	    cbbWindow.newcat
	    tkwait window .newcat
	    # whether or not the user adds it, they have skipped it, so skip it.
	    set cbb(splt_found) 1
	}
	focus [tk_focusNext $field]
	reset_split_memory
	return 
    }
    
    set cbb(splt_entry) 1
    set cbb(splt_found) 0
    set cbb(splt_partial) $cats($N)
    puts $eng "find_cat 0\t$cats($N)"; flush $eng
    gets $eng result
    gets $eng match 
    if { $cbb(debug) } { puts $result; puts $match}
    
    set pieces [split $match "\t"]
    set cats($N) [lindex $pieces 1]
    
    set pieces [split $result "\t"]
    set stat  [lindex $pieces 0]
    set value [lindex $pieces 1]
    
    switch $stat {
	"NONE" {
	    .status.line configure -text "no match found."
	    set cats($N) $cbb(splt_partial)
	} 
	"PARTIAL" {
	    set cbb(splt_found) 1
	    .status.line configure -text "partial match: '$value'"
	} 
	"UNIQUE" -
	"EXACT" {
	    set cbb(splt_found) 1
	    .status.line configure -text "exact match: '$value'"
	    set cbb(splt_match) $cats($N)
	    focus [tk_focusNext $field]
	    reset_split_memory
	    return
	}
	"COMPLETE" {
	    set cbb(splt_found) 1
	    .status.line configure -text "ambiguous match: '$value'"
	}
	default {
	    if { $cbb(debug) } { puts "error in cat lookup..." }
	    exit;
	}
    }
    tkEntrySetCursor $field end
    set cbb(splt_match) $cats($N)
    return
}


proc splitsLeavingAmt { field } {
    global cbb

    # just left an amt field

    set text "Sum = [splitsSum]"
    .splits.frame.total configure -text $text
    if { $cbb(debug) } { puts "Leaving amount field" }
}

proc splitsSum {} {
    global cbb debit credit cats amts

    set trans_total [expr $credit - $debit]

    set total 0
    set i 0
    while { $i < $cbb(max_splits) } {
        set amount $amts($i)
	set diff [format "%.2f" [expr $trans_total - $total]]
	if { "$amount" != "" } {
	    if { "$cats($i)" != "" } {
		if { $cbb(debug) } { puts "$i $amount" }
		set total [expr $total + $amount]
	    } else {
		if { $diff != 0.00 } {
		    set amts($i) $diff
		} else {
		    set amts($i) ""
		}
		set amount $diff
		set total [expr $total + $amount]
		if { $cbb(debug) } { puts "auto replacing difference with $diff" }
	    }
	} else {
	    if { [format "%.2f" $diff] != 0.00 } {
		set amts($i) $diff
		set amount $amts($i)
		set total [expr $total + $amount]
		if { $cbb(debug) } { puts "auto entering difference" }
	    }
	}
        incr i
    }

    return [format "%.2f" $total]
}
